@@ -1,6 +1,6 @@
 /*
  * ebusd - daemon for communication with eBUS heating systems.
- * Copyright (C) 2023-2024 John Baier <ebusd@ebusd.eu>
+ * Copyright (C) 2023-2026 John Baier <ebusd@ebusd.eu>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -264,7 +264,7 @@ static int parse_opt(int key, char *arg, const argParseOpt *parseOpt, struct opt
     break;
 
   // Message configuration options:
-  case 'c':  // --configpath=https://cfg.ebusd.eu/
+  case 'c':  // --configpath=https://ebus.github.io/
     if (arg == nullptr || arg[0] == 0 || strcmp("/", arg) == 0) {
       argParseError(parseOpt, "invalid configpath");
       return EINVAL;
