@@ -1,6 +1,6 @@
 /*
  * ebusd - daemon for communication with eBUS heating systems.
- * Copyright (C) 2014-2024 John Baier <ebusd@ebusd.eu>
+ * Copyright (C) 2014-2026 John Baier <ebusd@ebusd.eu>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -481,7 +481,7 @@ result_t ScanHelper::loadScanConfigFile(symbol_t address, string* relativeFile)
       }
       match += remain.length();
     }
-    if (match >= bestMatch) {
+    if (match > bestMatch || (match == bestMatch && name.length() > best.length())) {
       bestMatch = match;
       best = name;
       bestDefaults = defaults;
