@@ -1,6 +1,6 @@
 /*
  * ebusd - daemon for communication with eBUS heating systems.
- * Copyright (C) 2014-2024 John Baier <ebusd@ebusd.eu>
+ * Copyright (C) 2014-2026 John Baier <ebusd@ebusd.eu>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -954,7 +954,8 @@ void Message::dumpField(const string& fieldName, bool withConditions, OutputForm
 }
 
 void Message::decodeJson(bool leadingSeparator, bool appendDirectionCondition, bool withData,
-                         OutputFormat outputFormat, ostringstream* output) const {
+                         OutputFormat outputFormat, ostringstream* output,
+                         AddAttributes* addAttrs) const {
   outputFormat |= OF_JSON;
   if (leadingSeparator) {
     *output << ",\n";
@@ -986,6 +987,9 @@ void Message::decodeJson(bool leadingSeparator, bool appendDirectionCondition, b
       *output << ",\n    \"condition\": ";
       m_condition->dumpJson(output);
     }
+    if (addAttrs) {
+      addAttrs->addAttrsTo(this, output);
+    }
   }
   if (withData) {
     *output << ",\n    \"lastup\": " << setw(0) << dec << getLastUpdateTime();
@@ -2486,67 +2490,72 @@ result_t MessageMap::resolveCondition(void (*readMessageFunc)(Message* message),
 
 result_t MessageMap::executeInstructions(void (*readMessageFunc)(Message* message), ostringstream* log) {
   result_t overallResult = RESULT_OK;
-  vector<string> remove;
-  for (auto& it : m_instructions) {
-    auto instructions = it.second;
-    bool removeSingletons = false;
-    vector<Instruction*> remain;
-    for (const auto instruction : instructions) {
-      if (!m_addAll && removeSingletons && instruction->isSingleton()) {
-        delete instruction;
-        continue;
-      }
-      Condition* condition = instruction->getCondition();
-      bool execute = m_addAll || condition == nullptr;
-      if (!execute) {
-        string errorDescription;
-        result_t result = resolveCondition(instruction->isSingleton()?readMessageFunc:nullptr, condition,
-            &errorDescription);
-        if (result != RESULT_OK) {
-          overallResult = result;
-          *log << "error resolving condition for \"";
-          instruction->getDestination(log);
-          *log << "\": " << getResultCode(result);
-          if (!errorDescription.empty()) {
-            *log << " " << errorDescription;
-          }
-        } else if (condition->isTrue()) {
-          execute = true;
-        }
-      }
-      if (execute) {
-        if (!m_addAll && instruction->isSingleton()) {
-          removeSingletons = true;
-        }
-        result_t result = instruction->execute(this, log);
-        if (result != RESULT_OK) {
-          overallResult = result;
-        }
-        delete instruction;
-      } else {
-        remain.push_back(instruction);
-      }
-    }
-    if (removeSingletons && !remain.empty()) {
-      instructions = remain;
-      remain.clear();
+  size_t maxRounds = 3, cntBefore, cntAfter;
+  do {
+    cntBefore = cntAfter = m_instructions.size();
+    vector<string> remove;
+    for (auto& it : m_instructions) {
+      auto instructions = it.second;
+      bool removeSingletons = false;
+      vector<Instruction*> remain;
       for (const auto instruction : instructions) {
-        if (!instruction->isSingleton()) {
-          remain.push_back(instruction);
+        if (!m_addAll && removeSingletons && instruction->isSingleton()) {
+          delete instruction;
           continue;
         }
-        delete instruction;
+        Condition* condition = instruction->getCondition();
+        bool execute = m_addAll || condition == nullptr;
+        if (!execute) {
+          string errorDescription;
+          result_t result = resolveCondition(instruction->isSingleton()?readMessageFunc:nullptr, condition,
+              &errorDescription);
+          if (result != RESULT_OK) {
+            overallResult = result;
+            *log << "error resolving condition for \"";
+            instruction->getDestination(log);
+            *log << "\": " << getResultCode(result);
+            if (!errorDescription.empty()) {
+              *log << " " << errorDescription;
+            }
+          } else if (condition->isTrue()) {
+            execute = true;
+          }
+        }
+        if (execute) {
+          if (!m_addAll && instruction->isSingleton()) {
+            removeSingletons = true;
+          }
+          result_t result = instruction->execute(this, log);
+          if (result != RESULT_OK) {
+            overallResult = result;
+          }
+          delete instruction;
+        } else {
+          remain.push_back(instruction);
+        }
+      }
+      if (removeSingletons && !remain.empty()) {
+        instructions = remain;
+        remain.clear();
+        for (const auto instruction : instructions) {
+          if (!instruction->isSingleton()) {
+            remain.push_back(instruction);
+            continue;
+          }
+          delete instruction;
+        }
+      }
+      if (remain.empty()) {
+        remove.push_back(it.first);
+      } else {
+        it.second = remain;
       }
     }
-    if (remain.empty()) {
-      remove.push_back(it.first);
-    } else {
-      it.second = remain;
+    cntAfter = m_instructions.size();
+    for (const auto& it : remove) {
+      m_instructions.erase(it);
     }
-  }
-  for (const auto& it : remove) {
-    m_instructions.erase(it);
-  }
+  } while (overallResult == RESULT_OK && cntAfter>cntBefore && --maxRounds>0);
   return overallResult;
 }
 
@@ -2902,7 +2911,7 @@ Message* MessageMap::getNextPoll() {
   return ret;
 }
 
-void MessageMap::dump(bool withConditions, OutputFormat outputFormat, ostream* output) const {
+void MessageMap::dump(bool withConditions, OutputFormat outputFormat, ostream* output, AddAttributes* addAttrs) const {
   bool first = true;
   bool isJson = (outputFormat & OF_JSON) != 0;
   if (isJson) {
@@ -2935,7 +2944,7 @@ void MessageMap::dump(bool withConditions, OutputFormat outputFormat, ostream* o
         }
         if (isJson) {
           ostringstream str;
-          message->decodeJson(false, false, false, outputFormat, &str);
+          message->decodeJson(false, false, false, outputFormat, &str, addAttrs);
           string add = str.str();
           size_t pos = add.find('{');
           *output << "   {\n    \"circuit\": \"" << message->getCircuit() << "\", " << add.substr(pos+1);
