@@ -1,6 +1,6 @@
 /*
  * ebusd - daemon for communication with eBUS heating systems.
- * Copyright (C) 2014-2024 John Baier <ebusd@ebusd.eu>
+ * Copyright (C) 2014-2026 John Baier <ebusd@ebusd.eu>
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -355,6 +355,9 @@ int main() {
       {"x,,uch,,0x1-0x3", "2","10feffff0104", "00", "-rW:ERR: argument value out of valid range"},
       {"x,,uch", "\n     \"x\": {\"value\": 2}",    "10feffff0102", "00", "-jV", "\n     { \"name\": \"x\", \"slave\": false, \"type\": \"UCH\", \"isbits\": false, \"isadjustable\": false, \"isignored\": false, \"isreverse\": false, \"length\": 1, \"result\": \"number\", \"min\": 0, \"max\": 254, \"step\": 1, \"unit\": \"\", \"comment\": \"\"}"},
       {"x,,uch,,1-3:2", "\n     \"x\": {\"value\": 2}",    "10feffff0102", "00", "-jV", "\n     { \"name\": \"x\", \"slave\": false, \"type\": \"UCH\", \"isbits\": false, \"isadjustable\": false, \"isignored\": false, \"isreverse\": false, \"length\": 1, \"result\": \"number\", \"min\": 1, \"max\": 3, \"step\": 2, \"unit\": \"\", \"comment\": \"\"}"},
+      {"x,,u1l", "0",        "10feffff0100", "00", ""},
+      {"x,,u1l", "255",      "10feffff01ff", "00", ""},
+      {"x,,u1l", "-",        "10feffff01ff", "00", "Rw"},
       {"x,,sch", "-90",      "10feffff01a6", "00", ""},
       {"x,,sch", "0",        "10feffff0100", "00", ""},
       {"x,,sch", "-1",       "10feffff01ff", "00", ""},
@@ -372,6 +375,10 @@ int main() {
       {"x,,sch,,1-3", "4",    "10feffff0102", "00", "-Rw:ERR: argument value out of valid range"},
       {"x,,sch,,1-3", "2",    "10feffff0104", "00", "-rW:ERR: argument value out of valid range"},
       {"x,,sch,,-3--1", "-4", "10feffff01fe", "00", "-Rw:ERR: argument value out of valid range"},
+      {"x,,s1l", "0",        "10feffff0100", "00", ""},
+      {"x,,s1l", "127",      "10feffff017f", "00", ""},
+      {"x,,s1l", "-128",     "10feffff0180", "00", ""},
+      {"x,,s1l", "-",        "10feffff01ff", "00", "Rw"},
       {"x,,d1b", "-90",      "10feffff01a6", "00", ""},
       {"x,,d1b", "0",        "10feffff0100", "00", ""},
       {"x,,d1b", "-1",       "10feffff01ff", "00", ""},
@@ -415,6 +422,22 @@ int main() {
       {"x,,sir", "32767",    "10feffff027fff", "00", ""},
       {"x,,sir,10", "-9.0",  "10feffff02ffa6", "00", ""},
       {"x,,sir,-10", "-900", "10feffff02ffa6", "00", ""},
+      {"x,,s2l", "-90",      "10feffff02a6ff", "00", ""},
+      {"x,,s2l", "0",        "10feffff020000", "00", ""},
+      {"x,,s2l", "-1",       "10feffff02ffff", "00", ""},
+      {"x,,s2l", "-32768",   "10feffff020080", "00", ""},
+      {"x,,s2l", "-32767",   "10feffff020180", "00", ""},
+      {"x,,s2l", "32767",    "10feffff02ff7f", "00", ""},
+      {"x,,s2l,10", "-9.0",  "10feffff02a6ff", "00", ""},
+      {"x,,s2l,-10", "-900", "10feffff02a6ff", "00", ""},
+      {"x,,s2b", "-90",      "10feffff02ffa6", "00", ""},
+      {"x,,s2b", "0",        "10feffff020000", "00", ""},
+      {"x,,s2b", "-1",       "10feffff02ffff", "00", ""},
+      {"x,,s2b", "-32768",   "10feffff028000", "00", ""},
+      {"x,,s2b", "-32767",   "10feffff028001", "00", ""},
+      {"x,,s2b", "32767",    "10feffff027fff", "00", ""},
+      {"x,,s2b,10", "-9.0",  "10feffff02ffa6", "00", ""},
+      {"x,,s2b,-10", "-900", "10feffff02ffa6", "00", ""},
       {"x,,u3n", "38",       "10feffff03260000", "00", ""},
       {"x,,u3n", "0",        "10feffff03000000", "00", ""},
       {"x,,u3n", "16777214", "10feffff03feffff", "00", ""},
